package repos

import (
	"context"
	"gralias/models"

	"github.com/jmoiron/sqlx"
)

type CardMarksRepo interface {
	CardMarksByCardID(ctx context.Context, cardID uint32) ([]models.CardMark, error)
	CardMarksAdd(ctx context.Context, cm *models.CardMark) error
	CardMarksRemove(ctx context.Context, cardID uint32, username string) error
	CardMarksByRoomID(ctx context.Context, roomID string) ([]models.CardMark, error)
	CardMarksRemoveByRoomID(ctx context.Context, roomID string) error
}

func (r *RepoProvider) CardMarksByCardID(ctx context.Context, cardID uint32) ([]models.CardMark, error) {
	var cardMarks []models.CardMark
	err := sqlx.SelectContext(ctx, getDB(ctx, r.DB), &cardMarks, "SELECT * FROM card_marks WHERE card_id = ?", cardID)
	return cardMarks, err
}

func (r *RepoProvider) CardMarksAdd(ctx context.Context, cm *models.CardMark) error {
	_, err := getDB(ctx, r.DB).ExecContext(ctx, "INSERT INTO card_marks (card_id, username) VALUES (?, ?)", cm.CardID, cm.Username)
	return err
}

func (r *RepoProvider) CardMarksRemove(ctx context.Context, cardID uint32, username string) error {
	db := getDB(ctx, r.DB)
	_, err := db.ExecContext(ctx, "DELETE FROM card_marks WHERE card_id = ? AND username = ?", cardID, username)
	return err
}

func (r *RepoProvider) CardMarksByRoomID(ctx context.Context, roomID string) ([]models.CardMark, error) {
	var cardMarks []models.CardMark
	err := sqlx.SelectContext(ctx, getDB(ctx, r.DB), &cardMarks, "SELECT * FROM card_marks WHERE card_id IN (select id from word_cards where room_id = ?)", roomID)
	return cardMarks, err
}
func (r *RepoProvider) CardMarksRemoveByRoomID(ctx context.Context, roomID string) error {
	db := getDB(ctx, r.DB)
	_, err := db.ExecContext(ctx, "DELETE FROM card_marks WHERE room_id = ?;", roomID)
	return err
}
